import subprocess
import sys


def test_cli_version():
    # Try running the CLI with --version
    result = subprocess.run([
        sys.executable, "-m", "janito.cli.main_cli", "--version"
    ], capture_output=True, text=True)
    assert result.returncode == 0, f"Non-zero exit: {result.returncode}"
    assert "Janito" in result.stdout or "Janito" in result.stderr, "Version output missing 'Janito'"



from janito.drivers.openai.driver import OpenAIModelDriver

# Safe import of AzureOpenAI SDK
try:
    from openai import AzureOpenAI

    DRIVER_AVAILABLE = True
    DRIVER_UNAVAILABLE_REASON = None
except ImportError:
    DRIVER_AVAILABLE = False
    DRIVER_UNAVAILABLE_REASON = "Missing dependency: openai (pip install openai)"

from janito.llm.driver_config import LLMDriverConfig


class AzureOpenAIModelDriver(OpenAIModelDriver):
    def start(self, *args, **kwargs):
        # Ensure azure_deployment_name is set before starting
        config = getattr(self, 'config', None)
        deployment_name = None
        if config and hasattr(config, 'extra'):
            deployment_name = config.extra.get('azure_deployment_name')
        if not deployment_name:
            raise RuntimeError("AzureOpenAIModelDriver requires 'azure_deployment_name' to be set in config.extra['azure_deployment_name'] before starting.")
        # Call parent start if exists
        if hasattr(super(), 'start'):
            return super().start(*args, **kwargs)

    available = DRIVER_AVAILABLE
    unavailable_reason = DRIVER_UNAVAILABLE_REASON

    @classmethod
    def is_available(cls):
        return cls.available

    required_config = {"base_url"}  # Update key as used in your config logic

    def __init__(self, tools_adapter=None, provider_name=None):
        if not self.available:
            raise ImportError(
                f"AzureOpenAIModelDriver unavailable: {self.unavailable_reason}"
            )
        # Ensure proper parent initialization
        super().__init__(tools_adapter=tools_adapter, provider_name=provider_name)
        self.azure_endpoint = None
        self.api_version = None
        self.api_key = None

    def _prepare_api_kwargs(self, config, conversation):
        """
        Prepares API kwargs for Azure OpenAI, using the deployment name as the model parameter.
        Also ensures tool schemas are included if tools_adapter is present.
        """
        api_kwargs = super()._prepare_api_kwargs(config, conversation)
        deployment_name = config.extra.get("azure_deployment_name") if hasattr(config, "extra") else None
        if deployment_name:
            api_kwargs["model"] = deployment_name
        # Patch: Ensure tools are included for Azure as for OpenAI
        if self.tools_adapter:
            try:
                from janito.providers.openai.schema_generator import generate_tool_schemas
                tool_classes = self.tools_adapter.get_tool_classes()
                tool_schemas = generate_tool_schemas(tool_classes)
                api_kwargs["tools"] = tool_schemas
            except Exception as e:
                api_kwargs["tools"] = []
                if hasattr(config, "verbose_api") and config.verbose_api:
                    print(f"[AzureOpenAIModelDriver] Tool schema generation failed: {e}")
        return api_kwargs

    def _instantiate_openai_client(self, config):
        try:
            from openai import AzureOpenAI
            api_key_display = str(config.api_key)
            if api_key_display and len(api_key_display) > 8:
                api_key_display = api_key_display[:4] + "..." + api_key_display[-4:]
            client_kwargs = {
                "api_key": config.api_key,
                "azure_endpoint": getattr(config, "base_url", None),
                "api_version": config.extra.get("api_version", "2023-05-15"),
            }
            # Do NOT pass azure_deployment; deployment name is used as the 'model' param in API calls
            client = AzureOpenAI(**client_kwargs)
            return client
        except Exception as e:
            print(f"[ERROR] Exception during AzureOpenAI client instantiation: {e}", flush=True)
            import traceback
            print(traceback.format_exc(), flush=True)
            raise




import os
from pathlib import Path
import questionary
from questionary import Style
import os
from pathlib import Path
from prompt_toolkit.formatted_text import HTML
from prompt_toolkit import PromptSession
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.enums import EditingMode
from prompt_toolkit.formatted_text import HTML
from .prompt_style import chat_shell_style


"""
Profile selection logic for Janito Chat CLI using questionary.
"""

def _handle_helpful_assistant():
    return {"profile": "assistant", "profile_system_prompt": None}

def _handle_using_role():
    role_name = questionary.text("Enter the role name:").ask()
    return f"role:{role_name}"

def _get_toolbar(mode):
    if mode["multiline"]:
        return HTML("<b>Multiline mode (Esc+Enter to submit). Type /single to switch.</b>")
    else:
        return HTML("<b>Single-line mode (Enter to submit). Type /multi for multiline.</b>")

def _handle_custom_system_prompt():
    mode = {"multiline": False}
    bindings = KeyBindings()

    @bindings.add("c-r")
    def _(event):
        pass

    @bindings.add("f12")
    def _(event):
        buf = event.app.current_buffer
        buf.text = "Do It"
        buf.validate_and_handle()

    session = PromptSession(
        multiline=False,
        key_bindings=bindings,
        editing_mode=EditingMode.EMACS,
        bottom_toolbar=lambda: _get_toolbar(mode),
        style=chat_shell_style,
    )
    prompt_icon = HTML("<inputline>📝 </inputline>")
    while True:
        response = session.prompt(prompt_icon)
        if not mode["multiline"] and response.strip() == "/multi":
            mode["multiline"] = True
            session.multiline = True
            continue
        elif mode["multiline"] and response.strip() == "/single":
            mode["multiline"] = False
            session.multiline = False
            continue
        else:
            sanitized = response.strip()
            try:
                sanitized.encode("utf-8")
            except UnicodeEncodeError:
                sanitized = sanitized.encode("utf-8", errors="replace").decode("utf-8")
            return {"profile": None, "profile_system_prompt": sanitized}


def _load_user_profiles():
    user_profiles_dir = Path.home() / ".janito" / "profiles"
    profiles = {}
    if user_profiles_dir.exists() and user_profiles_dir.is_dir():
        for profile_file in user_profiles_dir.glob("*"):
            if profile_file.is_file():
                try:
                    with open(profile_file, "r", encoding="utf-8") as f:
                        profiles[profile_file.stem] = f.read().strip()
                except Exception:
                    # Ignore unreadable files
                    pass
    return profiles


def select_profile():
    user_profiles = _load_user_profiles()
    choices = [
        "helpful assistant",
        "developer",
        "software developer",
        "using role...",
        "full custom system prompt..."
    ]
    # Add user profiles to choices
    if user_profiles:
        choices.extend(user_profiles.keys())

    custom_style = Style([
        ("highlighted", "bg:#00aaff #ffffff"),  # background for item under cursor
        ("question", "fg:#00aaff bold"),
    ])
    answer = questionary.select(
        "Select a profile to use:",
        choices=choices,
        default=None,
        style=custom_style
    ).ask()

    if answer == "helpful assistant":
        return _handle_helpful_assistant()
    if answer == "using role...":
        return _handle_using_role()
    elif answer == "full custom system prompt...":
        return _handle_custom_system_prompt()
    elif answer in user_profiles:
        # Return the content of the user profile as a custom system prompt
        return {"profile": None, "profile_system_prompt": user_profiles[answer]}
    elif answer == "software developer":
        # Return the content of the built-in software developer profile prompt
        with open("./janito/agent/templates/profiles/system_prompt_template_software_developer.txt.j2", "r", encoding="utf-8") as f:
            prompt = f.read().strip()
        return {"profile": "software developer", "profile_system_prompt": prompt}
    return answer

    choices = [
        "helpful assistant",
        "developer",
        "using role...",
        "full custom system prompt..."
    ]
    custom_style = Style([
        ("highlighted", "bg:#00aaff #ffffff"),  # background for item under cursor
        ("question", "fg:#00aaff bold"),
    ])
    answer = questionary.select(
        "Select a profile to use:",
        choices=choices,
        default=None,
        style=custom_style
    ).ask()
    if answer == "helpful assistant":
        return _handle_helpful_assistant()
    if answer == "using role...":
        return _handle_using_role()
    elif answer == "full custom system prompt...":
        return _handle_custom_system_prompt()
    return answer
